@@ -1,41 +1,48 @@
-/*===========================================================================*\
+/* ========================================================================= *
  *                                                                           *
  *                               OpenMesh                                    *
- *      Copyright (C) 2001-2014 by Computer Graphics Group, RWTH Aachen      *
- *                           www.openmesh.org                                *
+ *           Copyright (c) 2001-2015, RWTH-Aachen University                 *
+ *           Department of Computer Graphics and Multimedia                  *
+ *                          All rights reserved.                             *
+ *                            www.openmesh.org                               *
  *                                                                           *
  *---------------------------------------------------------------------------*
- *  This file is part of OpenMesh.                                           *
+ * This file is part of OpenMesh.                                            *
+ *---------------------------------------------------------------------------*
  *                                                                           *
- *  OpenMesh is free software: you can redistribute it and/or modify         *
- *  it under the terms of the GNU Lesser General Public License as           *
- *  published by the Free Software Foundation, either version 3 of           *
- *  the License, or (at your option) any later version with the              *
- *  following exceptions:                                                    *
+ * Redistribution and use in source and binary forms, with or without        *
+ * modification, are permitted provided that the following conditions        *
+ * are met:                                                                  *
  *                                                                           *
- *  If other files instantiate templates or use macros                       *
- *  or inline functions from this file, or you compile this file and         *
- *  link it with other files to produce an executable, this file does        *
- *  not by itself cause the resulting executable to be covered by the        *
- *  GNU Lesser General Public License. This exception does not however       *
- *  invalidate any other reasons why the executable file might be            *
- *  covered by the GNU Lesser General Public License.                        *
+ * 1. Redistributions of source code must retain the above copyright notice, *
+ *    this list of conditions and the following disclaimer.                  *
  *                                                                           *
- *  OpenMesh is distributed in the hope that it will be useful,              *
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of           *
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
- *  GNU Lesser General Public License for more details.                      *
+ * 2. Redistributions in binary form must reproduce the above copyright      *
+ *    notice, this list of conditions and the following disclaimer in the    *
+ *    documentation and/or other materials provided with the distribution.   *
  *                                                                           *
- *  You should have received a copy of the GNU LesserGeneral Public          *
- *  License along with OpenMesh.  If not,                                    *
- *  see <http://www.gnu.org/licenses/>.                                      *
+ * 3. Neither the name of the copyright holder nor the names of its          *
+ *    contributors may be used to endorse or promote products derived from   *
+ *    this software without specific prior written permission.               *
  *                                                                           *
-\*===========================================================================*/
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS       *
+ * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED *
+ * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A           *
+ * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER *
+ * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,  *
+ * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,       *
+ * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR        *
+ * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF    *
+ * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING      *
+ * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS        *
+ * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              *
+ *                                                                           *
+ * ========================================================================= */
 
 /*===========================================================================*\
  *                                                                           *
- *   $Revision: 1053 $                                                         *
- *   $Date: 2014-05-09 14:44:18 +0200 (Fr, 09 Mai 2014) $                   *
+ *   $Revision$                                                         *
+ *   $Date$                   *
  *                                                                           *
 \*===========================================================================*/
 
@@ -46,7 +53,6 @@
 // OpenMesh
 #include <OpenMesh/Core/IO/reader/OBJReader.hh>
 #include <OpenMesh/Core/IO/IOManager.hh>
-#include <OpenMesh/Core/System/omstream.hh>
 #include <OpenMesh/Core/Utils/vector_cast.hh>
 #include <OpenMesh/Core/Utils/color_cast.hh>
 // STL
@@ -56,14 +62,14 @@
 #elif defined(_STLPORT_VERSION) && (_STLPORT_VERSION==0x460)
 #  include <cctype>
 #else
-#  include <cctype>
 using std::isspace;
 #endif
 
 #ifndef WIN32
-#include <string.h>
 #endif
 
+#include <fstream>
+
 //=== NAMESPACES ==============================================================
 
 
@@ -96,6 +102,18 @@ void trimString( std::string& _string) {
 
 //-----------------------------------------------------------------------------
 
+// remove duplicated indices from one face
+void remove_duplicated_vertices(BaseImporter::VHandles& _indices)
+{
+  BaseImporter::VHandles::iterator endIter = _indices.end();
+  for (BaseImporter::VHandles::iterator iter = _indices.begin(); iter != endIter; ++iter)
+    endIter = std::remove(iter+1, endIter, *(iter));
+
+  _indices.erase(endIter,_indices.end());
+}
+
+//-----------------------------------------------------------------------------
+
 _OBJReader_::
 _OBJReader_()
 {
@@ -147,6 +165,8 @@ read_material(std::fstream& _in)
   std::string keyWrd;
   std::string textureName;
 
+  std::stringstream  stream;
+
   std::string key;
   Material    mat;
   float       f1,f2,f3;
@@ -168,7 +188,8 @@ read_material(std::fstream& _in)
     if ( line.empty() )
       continue;
 
-    std::stringstream stream(line);
+    stream.str(line);
+    stream.clear();
 
     stream >> keyWrd;
 
@@ -270,16 +291,18 @@ read(std::istream& _in, BaseImporter& _bi, Options& _opt)
   std::string keyWrd;
 
   float                     x, y, z, u, v;
-  int                       r, g, b;
+  float                     r, g, b;
   BaseImporter::VHandles    vhandles;
   std::vector<Vec3f>        normals;
-  std::vector<Vec3uc>       colors;
+  std::vector<Vec3f>        colors;
   std::vector<Vec2f>        texcoords;
   std::vector<Vec2f>        face_texcoords;
   std::vector<VertexHandle> vertexHandles;
 
   std::string               matname;
 
+  std::stringstream         stream, lineData, tmp;
+
 
   // Options supplied by the user
   Options userOptions = _opt;
@@ -304,7 +327,8 @@ read(std::istream& _in, BaseImporter& _bi, Options& _opt)
       continue;
     }
 
-    std::stringstream stream(line);
+    stream.str(line);
+    stream.clear();
 
     stream >> keyWrd;
 
@@ -370,7 +394,7 @@ read(std::istream& _in, BaseImporter& _bi, Options& _opt)
         {
           if (  userOptions.vertex_has_color() ) {
             fileOptions += Options::VertexColor;
-            colors.push_back(OpenMesh::Vec3uc((unsigned char)r,(unsigned char)g,(unsigned char)b));
+            colors.push_back(OpenMesh::Vec3f(r,g,b));
           }
         }
       }
@@ -406,7 +430,7 @@ read(std::istream& _in, BaseImporter& _bi, Options& _opt)
 
       if ( !stream.fail()   ){
         if ( userOptions.vertex_has_color() ) {
-          colors.push_back(OpenMesh::Vec3uc((unsigned char)r,(unsigned char)g,(unsigned char)b));
+          colors.push_back(OpenMesh::Vec3f(r,g,b));
           fileOptions += Options::VertexColor;
         }
       }
@@ -438,7 +462,8 @@ read(std::istream& _in, BaseImporter& _bi, Options& _opt)
       // read full line after detecting a face
       std::string faceLine;
       std::getline(stream,faceLine);
-      std::stringstream lineData( faceLine );
+      lineData.str( faceLine );
+      lineData.clear();
 
       FaceHandle fh;
       BaseImporter::VHandles faceVertices;
@@ -459,7 +484,8 @@ read(std::istream& _in, BaseImporter& _bi, Options& _opt)
           if( found != std::string::npos ){
 
             // read the index value
-            std::stringstream tmp( vertex.substr(0,found) );
+            tmp.str( vertex.substr(0,found) );
+            tmp.clear();
 
             // If we get an empty string this property is undefined in the file
             if ( vertex.substr(0,found).empty() ) {
@@ -482,7 +508,8 @@ read(std::istream& _in, BaseImporter& _bi, Options& _opt)
           } else {
 
             // last component of the vertex, read it.
-            std::stringstream tmp( vertex );
+            tmp.str( vertex );
+            tmp.clear();
             tmp >> value;
 
             // Clear vertex after finished reading the line
@@ -575,7 +602,11 @@ read(std::istream& _in, BaseImporter& _bi, Options& _opt)
       // note that add_face can possibly triangulate the faces, which is why we have to
       // store the current number of faces first
       size_t n_faces = _bi.n_faces();
-      fh = _bi.add_face(faceVertices);
+      remove_duplicated_vertices(faceVertices);
+
+      //A minimum of three vertices are required.
+      if (faceVertices.size() > 2)
+        fh = _bi.add_face(faceVertices);
 
       if (!vhandles.empty() && fh.is_valid() )
         _bi.add_face_texcoords(fh, vhandles[0], face_texcoords);
